/**
 * 一维数组变二维数组
 * @param list
 * @param elementsPerSubArray
 * @returns {Array}
 */
function arrayToMatrix(list, elementsPerSubArray) {
    var matrix = [], i, k;

    for (i = 0, k = -1; i < list.length; i++) {
        if (i % elementsPerSubArray === 0) {
            k++;
            matrix[k] = [];
        }

        matrix[k].push(list[i]);
    }

    return matrix;
}

/**
 * 根据映射关系将数组变为对象
 */
function arr2Object(arr, mapArr) {
    var obj = {};
    for (var i = 0; i < mapArr.length; i++) {
        if(!arr[i]){
            //如果有空的就返回空
            return null;
        }
        obj[mapArr[i]]=arr[i];
    }
    return obj;
}

/**
 * 一个收入详情的字段
 */
INCOME_DETAIL_ARR=["type","name","unit","amount","price","taxRate","totalTaxInclude","totalTaxExclude"];

/**
 * 保存收入明细数据
 */
function saveIncomeDetail(url, data, projectId) {
    if(Array.isArray(data) && data.length != 0){
        var incomeDetails = convertIncomeDetail(data);
        var da = {
            "incomeDetails":incomeDetails,
            "projectId":projectId
        };
        console.log(da);
        $.ajax({
            url: url,
            data: JSON.stringify(da),
            type: "post",
            dataType: "json",
            contentType:"application/json",
            async: false,
            success: function (d) {
                console.log(d);
                updateIncomeData(incomeDetails);
            }
        });
    }
}

/**
 * 更新页面收入的数据【累加】
 */
function updateIncomeData(incomeDetails) {
    var deviceTaxInclude = 0;
    var deviceTaxExclude = 0;
    var engineerTaxInclude = 0;
    var engineerTaxExclude = 0;
    var serviceTaxInclude = 0;
    var serviceTaxExclude = 0;
    incomeDetails.forEach(function (t, number, ts) {
        if(t["type"] == "1"){
            //设备类
            deviceTaxInclude += parseFloat(t["totalTaxInclude"]);
            deviceTaxExclude += parseFloat(t["totalTaxExclude"]);
        }else if(t["type"] == "2"){
            //工程类
            engineerTaxInclude += parseFloat(t["totalTaxInclude"]);
            engineerTaxExclude += parseFloat(t["totalTaxExclude"]);
        }else if(t["type"] == "3"){
            //服务类
            serviceTaxInclude += parseFloat(t["totalTaxInclude"]);
            serviceTaxExclude += parseFloat(t["totalTaxExclude"]);
        }
    });
    $("input[name='incomeDeviceTaxInclude']").val(deviceTaxInclude);
    $("input[name='incomeDeviceTaxExclude']").val(deviceTaxExclude);
    $("input[name='incomeEngineerTaxInclude']").val(engineerTaxInclude);
    $("input[name='incomeEngineerTaxExclude']").val(engineerTaxExclude);
    $("input[name='incomeServiceTaxInclude']").val(serviceTaxInclude);
    $("input[name='incomeServiceTaxExclude']").val(serviceTaxExclude);

    $("input[name='incomeTotalTaxInclude']").val(deviceTaxInclude+engineerTaxInclude+serviceTaxInclude);
    $("input[name='incomeTotalTaxExclude']").val(deviceTaxExclude+engineerTaxExclude+serviceTaxExclude);
}


/**
 * 将页面收集到的数据转换为ajax请求的数据【一维数组转换为对象数组】
 * @param data
 * @returns {Array}
 */
function convertIncomeDetail(data) {
    var arr = arrayToMatrix(data,INCOME_DETAIL_ARR.length);

    var incomeDetails = [];
    for (var i=0;i<arr.length;i++){
        var obj = arr2Object(arr[i], INCOME_DETAIL_ARR);
        if(obj){
            incomeDetails.push(obj);
        }

    }
    return incomeDetails;
}

/**
 * 收集收入明细的数据，因为动态添加的行modal无法识别，所以自己来收集
 * @returns {Array}
 */
function collectIncomeData(className) {
    var a= [];
    $("." + className).each(function (t) {
        a.push($(this).val());
    });
    return a;
}

/**
 * 绑定每一行的删除事件，删除当前的一行tr
 */
function bindDeleteBtn() {
    $(".am-modal-line-delete").click(function () {
        //删除自己对应的tr
        $(this).parent().parent().remove();
    });
}

/**
 * 绑定每个可改变的输入框，修改后改变对应输入框的值
 */
function bindIncomeChangeableInput() {
    //数量改变
    $(".input-changeable-amount").change(function () {
        var amount = parseFloat($(this).val());
        //找到对应的单价和税率
        var price = parseFloat($(this).parent().parent().find(".input-changeable-price").val());
        var taxRate = parseFloat($(this).parent().parent().find(".input-changeable-tax-rate").val());
        console.log(amount,price,taxRate);

        $(this).parent().parent().find(".input-changeable-total-tax-include").val(amount*price);
        $(this).parent().parent().find(".input-changeable-total-tax-exclude").val(amount*price/(1+taxRate/100));
    });
    //单价改变
    $(".input-changeable-price").change(function () {
        var price = parseFloat($(this).val());
        //找到对应的数量和税率
        var amount = parseFloat($(this).parent().parent().find(".input-changeable-amount").val());
        var taxRate = parseFloat($(this).parent().parent().find(".input-changeable-tax-rate").val());
        console.log(amount,price,taxRate);

        $(this).parent().parent().find(".input-changeable-total-tax-include").val(amount*price);
        $(this).parent().parent().find(".input-changeable-total-tax-exclude").val(amount*price/(1+taxRate/100));
    });
    //税率改变
    $(".input-changeable-tax-rate").change(function () {
        var taxRate = parseFloat($(this).val());
        //找到对应的数量和单价
        var amount = parseFloat($(this).parent().parent().find(".input-changeable-amount").val());
        var price = parseFloat($(this).parent().parent().find(".input-changeable-price").val());
        console.log(amount,price,taxRate);

        $(this).parent().parent().find(".input-changeable-total-tax-exclude").val(amount*price/(1+taxRate/100));
    });
}